// cBackground.cpp

#include "cBackground.h"

#include <GL/gl.h>

#include <ctime>
#include <cstdio>
#include <memory.h>

using namespace std;

#include "psi3d/macros.h"

void cDomeBackground::drawBackground(float hour) {
    float t = ((hour * 360.0 / 24.0) - 90.0);
    float s = sin(t*0.017454)*0.5+0.5;
    vectorset(mUpColor, 0.0, 0.0, s*0.8+0.2);
    vectorset(mLoColor, s*0.95+0.05, s*0.95+0.05, s*0.95+0.05);
    vectorset(mBtColor, 0.4*s+0.1, 0.2*s+0.1, 0.1*s+0.1);
    
    GLint value;
    glGetIntegerv(GL_POLYGON_MODE, &value);
    
    if (value == GL_FILL) {
        //drawGround(0);
        drawUpperDome();
        drawLowerDome();
        drawMountains();
        //drawClouds(0);
    } else {
        drawMountains();
    }
}

/*
 * void cDomeBackground::drawClouds(GLuint tx)
 * {
 * glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT | GL_TEXTURE_BIT);
 * glDisable(GL_DEPTH_TEST);
 * glDisable(GL_LIGHTING);
 * glFogfv(GL_FOG_COLOR, mLoColor);
 *
 * // Get current Camera-Matrix.
 * float m[16];
 * glGetFloatv(GL_MODELVIEW_MATRIX, m);
 * // Position from 4th Col.
 * float p[3];
 * loop3i(p[i] = m[12+i]);
 * // Reset Position on 4th Col.
 * loop3i(m[12+i] = 0);
 * // Apply transpose-inverse to position.
 * transposeMatrix4(m);
 * applyMatrix2(m, p);
 * //printf("height over ground: %.2f\n", p[1]);
 * //printf("Position: %.2f %.2f\n", p[0], p[2]);
 *
 * glPushMatrix();
 * glLoadIdentity();
 * glMultTransposeMatrixf(m); // Transpose to undo previous transpose.
 * glTranslatef(0, p[Y], 0);
 *
 * clock_t c = clock();
 * float e = 20.0f * M_PI / 180.0f;
 * float h = 200.0f;
 * float t = 0.01f * (float(c) / CLOCKS_PER_SEC);
 * t = fmod(t, 1);
 *
 * glColor3fv(mLoColor);
 * glBindTexture(GL_TEXTURE_2D, tx);
 * float dx = 1.0f * t;
 * float dz = 1.0f * t;
 * glBegin(GL_TRIANGLE_FAN);
 * glTexCoord2f(dx, dz);
 * glVertex3f(0.0, h, 0.0);
 * for (int i=0; i<=18; i++) {
 * glTexCoord2f(cos(i*e)*5.0+dx, sin(i*e)*5.0+dz);
 * glVertex3f(cos(i*e)*2000.0, h, sin(i*e)*2000.0);
 * };
 * glEnd();
 *
 * glPopMatrix();
 * glPopAttrib();
 * }
 *
 *
 * void cDomeBackground::drawGround(GLuint tx)
 * {
 * glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT | GL_TEXTURE_BIT);
 * glDisable(GL_DEPTH_TEST);
 * glDisable(GL_LIGHTING);
 *
 * // Get current Camera-Matrix.
 * float m[16];
 * glGetFloatv(GL_MODELVIEW_MATRIX, m);
 * // Position from 4th Col.
 * float p[3];
 * loop3i(p[i] = m[12+i]);
 * // Reset Position on 4th Col.
 * loop3i(m[12+i] = 0);
 * // Apply transpose-inverse to position.
 * transposeMatrix4(m);
 * applyMatrix2(m, p);
 * //printf("height over ground: %.2f\n", p[1]);
 * //printf("Position: %.2f %.2f\n", p[0], p[2]);
 *
 * glPushMatrix();
 * glLoadIdentity();
 * glMultTransposeMatrixf(m); // Transpose to undo previous transpose.
 * glTranslatef(0, p[Y], 0);
 *
 * float dx = -p[0]/5.0;
 * float dz = -p[2]/5.0;
 * glColor3fv(mLoColor);
 * glBindTexture(GL_TEXTURE_2D, tx);
 *
 * glBegin(GL_TRIANGLE_FAN);
 * glNormal3f(0, 1, 0);
 * glTexCoord2f(dx, dz);
 * glVertex3f(0.0, 0.0, 0.0);
 * float e = 20.0*M_PI/180.0f;
 * for (int i=0; i<=18; i++) {
 * glNormal3f(0, 1, 0);
 * glTexCoord2f(cos(-i*e)*500.0+dx, sin(-i*e)*500.0+dz);
 * glVertex3f(cos(-i*e)*2500.0, 0, sin(-i*e)*2500.0);
 * };
 * glEnd();
 *
 * glPopMatrix();
 * glPopAttrib();
 * }
 */

void cDomeBackground::drawUpperDome() {
    glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT | GL_TEXTURE_BIT);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);
    
    // Load current Camera Matrix and
    // set Position (4th Column) = 0.
    float m[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, m);
    loop3i(m[12+i] = 0);
    
    glPushMatrix();
    glLoadIdentity();
    glMultMatrixf(m);
    glScalef(100, 100, 100);
    
    int vsteps = 8;
    float vf = 0.5f * M_PI / (float) vsteps;
    
    float color1[3];
    memcpy(color1, mUpColor, sizeof(float)*3);
    
    float color2[3];
    memcpy(color2, mLoColor, sizeof(float)*3);
    
    float slope[3];
    set3i(slope, (color2[i]-color1[i])/(float)vsteps);
    
    memcpy(color2, color1, sizeof(float)*3);
    vectoradd(color2, color2, slope);
    
    int hsteps = 18;
    float hf = 2 * M_PI / (float) hsteps;
    float h = 0;
    
    float s1 = sin(0);
    float c1 = cos(0);
    for (int i=1; i<=vsteps; i++) {
        float s2 = sin(i*vf);
        float c2 = cos(i*vf);
        glBegin(GL_TRIANGLE_STRIP);
        for (int j=0; j<=hsteps; j++) {
            float s = sin(h);
            float c = cos(h);
            glColor3fv(color1);
            glNormal3f(s1*c, c1, s1*s);
            glVertex3f(s1*c, c1, s1*s);
            glColor3fv(color2);
            glNormal3f(s2*c, c2, s2*s);
            glVertex3f(s2*c, c2, s2*s);
            h += hf;
        };
        glEnd();
        memcpy(color1, color2, sizeof(float)*3);
        vectoradd(color2, color2, slope);
        s1 = s2;
        c1 = c2;
    };
    
    glPopMatrix();
    glPopAttrib();
}


void cDomeBackground::drawLowerDome() {
    glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT | GL_TEXTURE_BIT);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);
    
    // Load current Camera Matrix and
    // set Position (4th Column) = 0.
    float m[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, m);
    loop3i(m[12+i] = 0);
    
    glPushMatrix();
    glLoadIdentity();
    glMultMatrixf(m);
    glScalef(100, 100, 100);
    
    int vsteps = 8;
    float vf = 0.5f * M_PI / (float) vsteps;
    
    const float f = 0.5;
    float color2[3] = { 0.4, 0.2, 0.1};
    memcpy(color2, mBtColor, sizeof(float)*3);
    float color1[3] = { color2[0]*f, color2[1]*f, color2[2]*f};
    
    float tmp[3];
    memcpy(tmp, color1, sizeof(float)*3);
    memcpy(color1, color2, sizeof(float)*3);
    memcpy(color2, tmp, sizeof(float)*3);
    
    float slope[3];
    set3i(slope, (color2[i]-color1[i])/(float)vsteps);
    
    memcpy(color2, color1, sizeof(float)*3);
    vectoradd(color2, color2, slope);
    
    int hsteps = 18;
    float hf = 2 * M_PI / (float) hsteps;
    float h = 0;
    float s1 = -sin(0);
    float c1 = -cos(0);
    for (int i=1; i<=vsteps; i++) {
        float s2 = -sin(i*vf);
        float c2 = -cos(i*vf);
        glBegin(GL_TRIANGLE_STRIP);
        for (int j=0; j<=hsteps; j++) {
            float s = sin(h);
            float c = cos(h);
            glColor3fv(color2);
            glNormal3f(s2*c, c2, s2*s);
            glVertex3f(s2*c, c2, s2*s);
            glColor3fv(color1);
            glNormal3f(s1*c, c1, s1*s);
            glVertex3f(s1*c, c1, s1*s);
            h += hf;
        };
        glEnd();
        memcpy(color1, color2, sizeof(float)*3);
        vectoradd(color2, color2, slope);
        s1 = s2;
        c1 = c2;
    };
    
    glPopMatrix();
    glPopAttrib();
}


void cDomeBackground::drawMountains() {
    glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT | GL_TEXTURE_BIT);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);
    
    // Load current Camera Matrix and
    // set Position (4th Column) = 0.
    float m[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, m);
    loop3i(m[12+i] = 0);
    
    glPushMatrix();
    glLoadIdentity();
    glMultMatrixf(m);
    glScalef(100, 100, 100);
    
    unsigned int r = 1;
    for (int i = 1; i < 6; i++) {
        float f = 1.0f / (1.0f + 0.3*i);
        int hsteps = 36 * i;
        float hf = 2 * M_PI / (float) hsteps;
        float h = 0;
        float u = 0;
        glBegin(GL_TRIANGLE_STRIP);
        glColor3f(0.3*f, 0.3*f, 0.4*f);
        for (int j=0; j<=hsteps; j++) {
            r = (r * 1234567) & ((1UL<<16)-1);
            u += 0.2 * ((r%100)*0.01 - 0.58);
            if (u < 0) u = 0;
            if (j == hsteps) u = 0;
            u *= f;
            float s = sin(h);
            float c = cos(h);
            glVertex3f(c, u, s);
            glVertex3f(c, 0, s);
            h += hf;
        }
        glEnd();
    };
    
    glPopMatrix();
    glPopAttrib();
}


