/**
 * File:   cController.h
 * Author: Benjamin Pickhardt (hackCraft.de)
 *
 * Created on March 28, 2008, 21:25 PM (1999)
 *
 * AI and autopilot stacked state machine for controlling entities.
 */

#ifndef _CONTROLLER_H
#define _CONTROLLER_H

class cController;

#include "cObject.h"

#include <string>
#include <stack>
using namespace std;


class cController {
    struct rInstruction {
        int opcode;
        tSerial entity;
        float v[3];
        int n;
    };
    
public:
    enum Opcodes { WAIT, ENGAGE, TARGET, ATTACK, FOLLOW, FLEE, GOTO, REPEAT };
    const char* getInstructionName(int opcode) {
        const char* names[] = { "WAIT", "ENGAGE", "TARGET", "ATTACK", "FOLLOW", "FLEE", "GOTO", "REPEAT" };
        return names[opcode];
    };
    
public:
    cObject* mDevice; // Controlled entity cObject.
    bool mEnabled; // Whether control is enabled (autopilot/ai enabled/disabled).
    
public:
    stack<rInstruction*> mInstructions;
    
public:
    cController(cObject* entity, bool enabled = true);
    virtual ~cController() {};
    
    void process();
    
    void clearAllInstructions();
    void newInstruction(int opcode, tSerial entity, float* vec = NULL, int n = 0);
    void oldInstruction();
    
    // Transition Functions
    void waitEnemy();
    void engageEnemy(tSerial entity);
    void targetEnemy(tSerial entity);
    void attackEnemy(tSerial entity);
    void followLeader(tSerial entity);
    void fleeEnemy(tSerial entity);
    void gotoDestination(float* v);
    void repeatInstructions(int n);

    // Use these from controlled Object (mDevice):
    // Queries - ask for objects
    // Fuzzy Predicates [0,1] - ask for fulfillment
    // Output Actions - ask for taking action
};


#endif
