/**
 * File:   cWeapon.h
 * Author: Benjamin Pickhardt (hackCraft.de)
 *
 * Created on March 5, 2008, 4:09 PM
 */

#ifndef _CWEAPON_H
#define	_CWEAPON_H

#include <list>
using namespace std;

class cWorld;
class cObject;


class cWeapon {
public:
    // Object whielding the weapon (normaly not damaged by weapon).
    cObject* mOwner;
    
    // Points To Current Vehicle Base Matrix.
    float* mBasefv;
    
    // Mounting Base Matrix relative to Vehicle Base Matrix.
    float* mMountfv;
    
    // Remaining reload time until next firing is possible.
    float mReloading;
    
public:
    cWeapon();
    cWeapon(cWeapon *source);
    
    // "Eignungsbeurteilung"
    //
    // Fuzzy-Value [0,1] indicating usability or fittness
    // of this weapon to be fired right now (not considering weapon state).
    // May sum up (depends on Weapon) (future):
    // +Ability/Possibility to hit target (may roll some dice).
    // +Possible damage on target.
    // +Possible damage on other enemy units (owner's enemies).
    // -Possible self-damage.
    // -Possible ally-damage.
    // +In (optimal) Firerange.
    // -Out of (optimal) Firerange.
    virtual float fit(cObject* target) { return 0.5f; };
    
    // Considering weapon state, true when weapon is
    // at hand, ready-loaded and unlocked.
    virtual bool ready() { return !(mReloading > +0.0f); };
    
    // Fire Weapon at target.
    virtual void fire(cObject* target) = 0;
    
    // Animation, transformation and rendering just like in cObject.
    // But called by cObject not by cWorld.
    virtual void animate(float spf) {};
    virtual void transform() {};
    virtual void drawSolid() {};
    virtual void drawEffect() {};
    virtual void drawHUD() {};
    
    // Utility to damage along a cylinder from the current position
    // of a projectile to the last position.
    // Returns amount of damaged objects.
    int damageAllByCylinder(float* position, float* velocity, float spf, float radius, int roles, float damage);
};


class cMachineGun : public cWeapon {
protected:
    struct Shrapnel {
        float pos[3];
        float vel[3];
        float cnt;
    };
    list<Shrapnel*> shrapnels;
    list<Shrapnel*> castoff;
    
public:
    cMachineGun();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid() {};
    virtual void drawEffect();
    virtual void drawHUD();
};


class cPlasmaGun : public cWeapon {
protected:
    struct Shrapnel {
        float pos[3];
        float vel[3];
        float cnt;
    };
    list<Shrapnel*> shrapnels;
    
public:
    cPlasmaGun();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid() {};
    virtual void drawEffect();
    virtual void drawHUD();
};


class cSparkGun : public cWeapon {
protected:
    struct Shrapnel {
        float pos[3];
        float vel[3];
        float cnt;
    };
    list<Shrapnel*> shrapnels;
    
public:
    cSparkGun();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid() {};
    virtual void drawEffect();
};


class cRaybeam : public cWeapon {
protected:
    float raylife;
    
public:
    cRaybeam();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid();
    virtual void drawEffect();
    virtual void drawHUD();
};


class cHomingMissile : public cWeapon {
protected:
    struct Smoke {
        float pos[3];
        float cnt;
        float timer;
    };
    struct Missile {
        float pos[3];
        float vel[3];
        float fuel; // remaining fuel in seconds
        float timer; // time since start
        float nextsmoke; // Countdown til spawning next Smoke-Particle.
        cObject* target;
        list<Smoke*> trail;
    };
    list<Missile*> missiles;
    
public:
    cHomingMissile();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid();
    virtual void drawEffect();
};


class cExplosion : public cWeapon {
protected:
    struct Smoke {
        float pos[3];
        float cnt;
        float timer;
    };
    struct Missile {
        float pos[3];
        float vel[3];
        float fuel; // remaining fuel in seconds
        float timer; // time since start
        float nextsmoke; // Countdown til spawning next Smoke-Particle.
        cObject* target;
        list<Smoke*> trail;
    };
    list<Missile*> missiles;
    
public:
    cExplosion();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid();
    virtual void drawEffect();
};


#endif	/* _CWEAPON_H */

